@@ -27,7 +27,6 @@ except:
 
 import libcloud
 
-
 from libcloud.utils.py3 import PY3
 from libcloud.utils.py3 import httplib
 from libcloud.utils.py3 import urlparse
@@ -290,15 +289,18 @@ class LoggingConnection():
             cmd.extend(["--data-binary", pquote(body)])
 
         cmd.extend(["--compress"])
-
-        cmd.extend([pquote("https://%s:%d%s" % (self.host, self.port, url))])
+        cmd.extend([pquote("%s://%s:%d%s" % (self.protocol, self.host,
+                                             self.port, url))])
         return " ".join(cmd)
 
+
 class LoggingHTTPSConnection(LoggingConnection, LibcloudHTTPSConnection):
     """
     Utility Class for logging HTTPS connections
     """
 
+    protocol = 'https'
+
     def getresponse(self):
         r = LibcloudHTTPSConnection.getresponse(self)
         if self.log is not None:
@@ -317,11 +319,14 @@ class LoggingHTTPSConnection(LoggingConnection, LibcloudHTTPSConnection):
         return LibcloudHTTPSConnection.request(self, method, url, body,
                                                headers)
 
+
 class LoggingHTTPConnection(LoggingConnection, LibcloudHTTPConnection):
     """
     Utility Class for logging HTTP connections
     """
 
+    protocol = 'http'
+
     def getresponse(self):
         r = LibcloudHTTPConnection.getresponse(self)
         if self.log is not None:
@@ -425,9 +430,11 @@ class Connection(object):
         secure = self.secure
 
         if getattr(self, 'base_url', None) and base_url == None:
-            (host, port, secure, request_path) = self._tuple_from_url(self.base_url)
+            (host, port,
+             secure, request_path) = self._tuple_from_url(self.base_url)
         elif base_url != None:
-            (host, port, secure, request_path) = self._tuple_from_url(base_url)
+            (host, port,
+             secure, request_path) = self._tuple_from_url(base_url)
         else:
             host = host or self.host
             port = port or self.port
@@ -453,8 +460,8 @@ class Connection(object):
         """
         Append a token to a user agent string.
 
-        Users of the library should call this to uniquely identify thier requests
-        to a provider.
+        Users of the library should call this to uniquely identify thier
+        requests to a provider.
 
         @type token: C{str}
         @param token: Token to add to the user agent.
@@ -608,6 +615,7 @@ class Connection(object):
         """
         return data
 
+
 class PollingConnection(Connection):
     """
     Connection class which can also work with the async APIs.
@@ -733,19 +741,23 @@ class ConnectionKey(Connection):
         Initialize `user_id` and `key`; set `secure` to an C{int} based on
         passed value.
         """
-        super(ConnectionKey, self).__init__(secure=secure, host=host, port=port, url=url)
+        super(ConnectionKey, self).__init__(secure=secure, host=host,
+                                            port=port, url=url)
         self.key = key
 
+
 class ConnectionUserAndKey(ConnectionKey):
     """
-    Base connection which accepts a user_id and key
+    Base connection which accepts a user_id and key.
     """
 
     user_id = None
 
-    def __init__(self, user_id, key, secure=True, host=None, port=None, url=None):
+    def __init__(self, user_id, key, secure=True,
+                 host=None, port=None, url=None):
         super(ConnectionUserAndKey, self).__init__(key, secure=secure,
-                                                   host=host, port=port, url=url)
+                                                   host=host, port=port,
+                                                   url=url)
         self.user_id = user_id
 
 
@@ -757,7 +769,7 @@ class BaseDriver(object):
     connectionCls = ConnectionKey
 
     def __init__(self, key, secret=None, secure=True, host=None, port=None,
-                 api_version=None):
+                 api_version=None, **kwargs):
         """
         @keyword    key:    API key or username to used
         @type       key:    str
@@ -796,6 +808,8 @@ class BaseDriver(object):
         if port is not None:
             args.append(port)
 
+        self.api_version = api_version
+
         self.connection = self.connectionCls(*args, **self._ex_connection_class_kwargs())
 
         self.connection.driver = self
