@@ -66,6 +66,7 @@ __all__ = [
     "LibcloudHTTPConnection"
     ]
 
+
 class UuidMixin(object):
     """
     Mixin class for get_uuid function.
@@ -80,7 +81,7 @@ class UuidMixin(object):
         @return: C{string}
 
         The hash is a function of an SHA1 hash of the node, node image,
-        or node size's ID and its driver which means that it should be 
+        or node size's ID and its driver which means that it should be
         unique between all objects of its type.
         In some subclasses (e.g. GoGridNode) there is no ID
         available so the public IP address is used.  This means that,
@@ -391,10 +392,10 @@ class NodeDriver(BaseDriver):
     NODE_STATE_MAP = {}
 
     def __init__(self, key, secret=None, secure=True, host=None, port=None,
-                 api_version=None):
+                 api_version=None, **kwargs):
         super(NodeDriver, self).__init__(key=key, secret=secret, secure=secure,
                                          host=host, port=port,
-                                         api_version=api_version)
+                                         api_version=api_version, **kwargs)
 
     def create_node(self, **kwargs):
         """Create a new node instance.
@@ -524,6 +525,10 @@ class NodeDriver(BaseDriver):
                              to attempt to authenticate. (optional)
         @type       ssh_key: C{string} or C{list} of C{string}s
 
+        @keyword    max_tries: How many times to retry if a deployment fails
+                               before giving up (default is 3)
+        @type       max_tries: C{int}
+
         See L{NodeDriver.create_node} for more keyword args.
 
         >>> from libcloud.compute.drivers.dummy import DummyNodeDriver
@@ -567,6 +572,7 @@ class NodeDriver(BaseDriver):
                 password = kwargs['auth'].password
 
         node = self.create_node(**kwargs)
+        max_tries = kwargs.get('max_tries', 3)
 
         if 'generates_password' in self.features['create_node']:
             password = node.extra.get('password')
@@ -595,7 +601,7 @@ class NodeDriver(BaseDriver):
             self._run_deployment_script(task=kwargs['deploy'],
                                         node=node,
                                         ssh_client=ssh_client,
-                                        max_tries=3)
+                                        max_tries=max_tries)
         except Exception:
             e = sys.exc_info()[1]
             raise DeploymentError(node, e)
@@ -704,8 +710,9 @@ class NodeDriver(BaseDriver):
             except Exception:
                 tries += 1
                 if tries >= max_tries:
-                    raise LibcloudError(value='Failed after %d tries'
-                                        % (max_tries), driver=self)
+                    e = sys.exc_info()[1]
+                    raise LibcloudError(value='Failed after %d tries: %s'
+                                        % (max_tries, str(e)), driver=self)
             else:
                 ssh_client.close()
                 return node
