@@ -27,6 +27,8 @@ import os
 
 import warnings
 
+from itertools import chain
+
 from libcloud.utils.py3 import httplib
 from libcloud.utils.py3 import b
 from libcloud.utils.py3 import next
@@ -36,8 +38,8 @@ import base64
 
 from xml.etree import ElementTree as ET
 
-from libcloud.common.openstack import OpenStackBaseConnection
-from libcloud.common.types import MalformedResponseError
+from libcloud.common.openstack import OpenStackBaseConnection, OpenStackDriverMixin
+from libcloud.common.types import MalformedResponseError, LibcloudError
 from libcloud.compute.types import NodeState, Provider
 from libcloud.compute.base import NodeSize, NodeImage
 from libcloud.compute.base import NodeDriver, Node, NodeLocation
@@ -107,8 +109,13 @@ class OpenStackResponse(Response):
             text = "; ".join([err.text or '' for err in body.getiterator()
                               if err.text])
         elif self.has_content_type('application/json'):
-            text = ';'.join([fault_data['message'] for fault_data
-                             in body.values()])
+            values = body.values()
+
+            if len(values) > 0 and 'message' in values[0]:
+                text = ';'.join([fault_data['message'] for fault_data
+                                 in values])
+            else:
+                text = body
         else:
             # while we hope a response is always one of xml or json, we have
             # seen html or text in the past, its not clear we can really do
@@ -120,6 +127,10 @@ class OpenStackResponse(Response):
 
 
 class OpenStackComputeConnection(OpenStackBaseConnection):
+    # default config for http://devstack.org/
+    service_type = 'compute'
+    service_name = 'nova'
+    service_region = 'RegionOne'
 
     def request(self, action, params=None, data='', headers=None,
                 method='GET'):
@@ -132,7 +143,7 @@ class OpenStackComputeConnection(OpenStackBaseConnection):
             headers = {'Content-Type': self.default_content_type}
 
         if method == "GET":
-            params['cache-busting'] = binascii.hexlify(os.urandom(8))
+            self._add_cache_busting_to_params(params)
 
         return super(OpenStackComputeConnection, self).request(
             action=action,
@@ -140,7 +151,7 @@ class OpenStackComputeConnection(OpenStackBaseConnection):
             method=method, headers=headers)
 
 
-class OpenStackNodeDriver(NodeDriver):
+class OpenStackNodeDriver(NodeDriver, OpenStackDriverMixin):
     """
     Base OpenStack node driver. Should not be used directly.
     """
@@ -177,6 +188,10 @@ class OpenStackNodeDriver(NodeDriver):
                     (api_version))
         return super(OpenStackNodeDriver, cls).__new__(cls)
 
+    def __init__(self, *args, **kwargs):
+        OpenStackDriverMixin.__init__(self, **kwargs)
+        super(OpenStackNodeDriver, self).__init__(*args, **kwargs)
+
     def destroy_node(self, node):
         uri = '/servers/%s' % (node.id)
         resp = self.connection.request(uri, method='DELETE')
@@ -206,14 +221,7 @@ class OpenStackNodeDriver(NodeDriver):
         return [NodeLocation(0, '', '', self)]
 
     def _ex_connection_class_kwargs(self):
-        rv = {}
-        if self._ex_force_base_url:
-            rv['ex_force_base_url'] = self._ex_force_base_url
-        if self._ex_force_auth_url:
-            rv['ex_force_auth_url'] = self._ex_force_auth_url
-        if self._ex_force_auth_version:
-            rv['ex_force_auth_version'] = self._ex_force_auth_version
-        return rv
+        return self.openstack_connection_kwargs()
 
     def ex_get_node_details(self, node_id):
         # @TODO: Remove this if in 0.6
@@ -245,7 +253,6 @@ class OpenStack_1_0_Response(OpenStackResponse):
 
 class OpenStack_1_0_Connection(OpenStackComputeConnection):
     responseCls = OpenStack_1_0_Response
-    _url_key = "server_url"
     default_content_type = 'application/xml; charset=UTF-8'
     accept_format = 'application/xml'
     XML_NAMESPACE = 'http://docs.rackspacecloud.com/servers/api/v1.0'
@@ -269,9 +276,6 @@ class OpenStack_1_0_NodeDriver(OpenStackNodeDriver):
     features = {"create_node": ["generates_password"]}
 
     def __init__(self, *args, **kwargs):
-        self._ex_force_base_url = kwargs.pop('ex_force_base_url', None)
-        self._ex_force_auth_url = kwargs.pop('ex_force_auth_url', None)
-        self._ex_force_auth_version = kwargs.pop('ex_force_auth_version', None)
         self._ex_force_api_version = str(kwargs.pop('ex_force_api_version',
                                                     None))
         self.XML_NAMESPACE = self.connectionCls.XML_NAMESPACE
@@ -294,7 +298,9 @@ class OpenStack_1_0_NodeDriver(OpenStackNodeDriver):
                              'created': element.get('created'),
                              'status': element.get('status'),
                              'serverId': element.get('serverId'),
-                             'progress': element.get('progress')})
+                             'progress': element.get('progress'),
+                             'minDisk': element.get('minDisk'),
+                             'minRam': element.get('minRam')})
 
     def _change_password_or_name(self, node, name=None, password=None):
         uri = '/servers/%s' % (node.id)
@@ -717,13 +723,14 @@ class OpenStack_1_0_NodeDriver(OpenStackNodeDriver):
                                       servers=servers)
 
     def _to_ip_addresses(self, el):
-        return OpenStack_1_0_NodeIpAddresses(
-            [ip.get('addr') for ip in
-             findall(findall(el, 'public', self.XML_NAMESPACE)[0],
-             'ip', self.XML_NAMESPACE)],
-            [ip.get('addr') for ip in
-             findall(findall(el, 'private', self.XML_NAMESPACE)[0],
-             'ip', self.XML_NAMESPACE)])
+        public_ips = [ip.get('addr') for ip in findall(findall(el, 'public',
+                                                 self.XML_NAMESPACE)[0],
+                                                 'ip', self.XML_NAMESPACE)]
+        private_ips = [ip.get('addr') for ip in findall(findall(el, 'private',
+                                                 self.XML_NAMESPACE)[0],
+                                                 'ip', self.XML_NAMESPACE)]
+
+        return OpenStack_1_0_NodeIpAddresses(public_ips, private_ips)
 
     def _get_size_price(self, size_id):
         try:
@@ -766,7 +773,6 @@ class OpenStack_1_1_Response(OpenStackResponse):
 
 class OpenStack_1_1_Connection(OpenStackComputeConnection):
     responseCls = OpenStack_1_1_Response
-    _url_key = "server_url"
     accept_format = 'application/json'
     default_content_type = 'application/json; charset=UTF-8'
 
@@ -786,10 +792,6 @@ class OpenStack_1_1_NodeDriver(OpenStackNodeDriver):
     features = {"create_node": ["generates_password"]}
 
     def __init__(self, *args, **kwargs):
-        self._ex_force_base_url = kwargs.pop('ex_force_base_url', None)
-        self._ex_force_auth_url = kwargs.pop('ex_force_auth_url', None)
-        self._ex_force_auth_version = kwargs.pop('ex_force_auth_version',
-                                                 None)
         self._ex_force_api_version = str(kwargs.pop('ex_force_api_version',
                                                     None))
         super(OpenStack_1_1_NodeDriver, self).__init__(*args, **kwargs)
@@ -829,6 +831,7 @@ class OpenStack_1_1_NodeDriver(OpenStackNodeDriver):
         return images
 
     def _to_image(self, api_image):
+        server = api_image.get('server', {})
         return NodeImage(
                       id=api_image['id'],
                       name=api_image['name'],
@@ -839,6 +842,9 @@ class OpenStack_1_1_NodeDriver(OpenStackNodeDriver):
                                  status=api_image['status'],
                                  progress=api_image.get('progress'),
                                  metadata=api_image.get('metadata'),
+                                 serverId=server.get('id'),
+                                 minDisk=api_image.get('minDisk'),
+                                 minRam=api_image.get('minRam'),
                       )
                   )
 
@@ -1023,7 +1029,8 @@ class OpenStack_1_1_NodeDriver(OpenStackNodeDriver):
             state=self.NODE_STATE_MAP.get(api_node['status'],
                                           NodeState.UNKNOWN),
             public_ips=[addr_desc['addr'] for addr_desc in
-                       api_node['addresses'].get('public', [])],
+                       chain(api_node['addresses'].get('public', []),
+                           api_node['addresses'].get('internet', []))],
             private_ips=[addr_desc['addr'] for addr_desc in
                         api_node['addresses'].get('private', [])],
             driver=self,
@@ -1038,6 +1045,8 @@ class OpenStack_1_1_NodeDriver(OpenStackNodeDriver):
                      link['rel'] == 'self'),
                 metadata=api_node['metadata'],
                 password=api_node.get('adminPass'),
+                created=api_node['created'],
+                updated=api_node['updated'],
             ),
         )
 
