@@ -30,6 +30,8 @@ __all__ = [
     "InvalidCredsError",
     "InvalidCredsException"
     ]
+
+
 class Provider(object):
     """
     Defines for each of the supported providers
@@ -59,6 +61,8 @@ class Provider(object):
     @cvar: EC2_US_WEST_OREGON: Amazon AWS US West 2 (Oregon)
     @cvar CLOUDSTACK: CloudStack
     @cvar CLOUDSIGMA_US: CloudSigma US Las Vegas
+    @cvar RACKSPACE_NOVA_BETA: Rackspace Nova Private Beta (ORD)
+    @cvar LIBVIRT: Libvirt driver
     """
     DUMMY = 0
     EC2 = 1  # deprecated name
@@ -102,6 +106,12 @@ class Provider(object):
     CLOUDSTACK = 37
     CLOUDSIGMA_US = 38
     EC2_SA_EAST = 39
+    RACKSPACE_NOVA_BETA = 40
+    RACKSPACE_NOVA_DFW = 41
+    LIBVIRT = 42
+    ELASTICHOSTS_US2 = 43
+    ELASTICHOSTS_CA1 = 44
+
 
 class NodeState(object):
     """
@@ -119,6 +129,7 @@ class NodeState(object):
     PENDING = 3
     UNKNOWN = 4
 
+
 class Architecture(object):
     """
     Image and size architectures.
@@ -129,6 +140,7 @@ class Architecture(object):
     I386 = 0
     X86_X64 = 1
 
+
 class DeploymentError(LibcloudError):
     """
     Exception used when a Deployment Task failed.
@@ -139,8 +151,6 @@ class DeploymentError(LibcloudError):
         self.node = node
         self.value = original_exception
 
-    def __str__(self):
-        return repr(self.value)
 
 """Deprecated alias of L{DeploymentException}"""
 DeploymentException = DeploymentError
