@@ -13,7 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from libcloud.utils import get_driver as get_provider_driver
+from libcloud.utils.misc import get_driver as get_provider_driver
 from libcloud.dns.types import Provider
 
 DRIVERS = {
@@ -26,7 +26,9 @@ DRIVERS = {
     Provider.RACKSPACE_US:
         ('libcloud.dns.drivers.rackspace', 'RackspaceUSDNSDriver'),
     Provider.RACKSPACE_UK:
-        ('libcloud.dns.drivers.rackspace', 'RackspaceUKDNSDriver')
+        ('libcloud.dns.drivers.rackspace', 'RackspaceUKDNSDriver'),
+    Provider.ROUTE_53:
+        ('libcloud.dns.drivers.route_53', 'Route53DNSDriver')
 }
 
 
