@@ -29,7 +29,7 @@ from libcloud.loadbalancer.base import DEFAULT_ALGORITHM
 from libcloud.common.types import LibcloudError
 from libcloud.common.base import JsonResponse, PollingConnection
 from libcloud.loadbalancer.types import State, MemberCondition
-from libcloud.common.openstack import OpenStackBaseConnection
+from libcloud.common.openstack import OpenStackBaseConnection, OpenStackDriverMixin
 from libcloud.common.rackspace import (
         AUTH_URL_US, AUTH_URL_UK)
 
@@ -226,15 +226,16 @@ class RackspaceAccessRule(object):
 class RackspaceConnection(OpenStackBaseConnection, PollingConnection):
     responseCls = RackspaceResponse
     auth_url = AUTH_URL_US
-    _url_key = "lb_url"
     poll_interval = 2
     timeout = 80
 
-    def __init__(self, user_id, key, secure=True, **kwargs):
+    def __init__(self, user_id, key, secure=True, ex_force_region='ord',
+                 **kwargs):
         super(RackspaceConnection, self).__init__(user_id, key, secure,
                                                   **kwargs)
         self.api_version = 'v1.0'
         self.accept_format = 'application/json'
+        self._ex_force_region = ex_force_region
 
     def request(self, action, params=None, data='', headers=None,
                 method='GET'):
@@ -246,7 +247,7 @@ class RackspaceConnection(OpenStackBaseConnection, PollingConnection):
         if method in ('POST', 'PUT'):
             headers['Content-Type'] = 'application/json'
         if method == 'GET':
-            params['cache-busing'] = binascii.hexlify(os.urandom(8))
+            self._add_cache_busting_to_params(params)
 
         return super(RackspaceConnection, self).request(action=action,
                 params=params, data=data, method=method, headers=headers)
@@ -263,12 +264,40 @@ class RackspaceConnection(OpenStackBaseConnection, PollingConnection):
 
         return state == 'ACTIVE'
 
+    def get_endpoint(self):
+        """
+        FIXME:
+        Dirty, dirty hack. Loadbalancers so not show up in the auth 1.1 service
+        catalog, so we build it from the servers url.
+        """
+
+        if self._auth_version == "1.1":
+            ep = self.service_catalog.get_endpoint(name="cloudServers")
+
+            return self._construct_loadbalancer_endpoint_from_servers_endpoint(ep)
+        elif "2.0" in self._auth_version:
+            ep = self.service_catalog.get_endpoint(name="cloudServers",
+                service_type="compute",
+                region=None)
+
+            return self._construct_loadbalancer_endpoint_from_servers_endpoint(ep)
+        else:
+            raise LibcloudError("Auth version %s not supported" % \
+                self._auth_version)
+
+    def _construct_loadbalancer_endpoint_from_servers_endpoint(self, ep):
+        if 'publicURL' in ep:
+            loadbalancer_prefix = "%s.loadbalancers" % self._ex_force_region
+            return ep['publicURL'].replace("servers", loadbalancer_prefix)
+        else:
+            raise LibcloudError('Could not find specified endpoint')
+
 
 class RackspaceUKConnection(RackspaceConnection):
     auth_url = AUTH_URL_UK
 
 
-class RackspaceLBDriver(Driver):
+class RackspaceLBDriver(Driver, OpenStackDriverMixin):
     connectionCls = RackspaceConnection
     api_name = 'rackspace_lb'
     name = 'Rackspace LB'
@@ -300,6 +329,18 @@ class RackspaceLBDriver(Driver):
 
     _ALGORITHM_TO_VALUE_MAP = reverse_dict(_VALUE_TO_ALGORITHM_MAP)
 
+    def __init__(self, *args, **kwargs):
+        OpenStackDriverMixin.__init__(self, *args, **kwargs)
+        self._ex_force_region = kwargs.pop('ex_force_region', None)
+        super(RackspaceLBDriver, self).__init__(*args, **kwargs)
+
+    def _ex_connection_class_kwargs(self):
+        kwargs = self.openstack_connection_kwargs()
+        if self._ex_force_region:
+            kwargs['ex_force_region'] = self._ex_force_region
+
+        return kwargs
+
     def list_protocols(self):
         return self._to_protocols(
                    self.connection.request('/loadbalancers/protocols').object)
@@ -331,24 +372,65 @@ class RackspaceLBDriver(Driver):
 
     def create_balancer(self, name, members, protocol='http',
                         port=80, algorithm=DEFAULT_ALGORITHM):
+        return self.ex_create_balancer(name, members, protocol, port,
+                                       algorithm)
+
+    def ex_create_balancer(self, name, members, protocol='http',
+                           port=80, algorithm=DEFAULT_ALGORITHM, vip='PUBLIC'):
+        """
+        Creates a new load balancer instance
+
+        @keyword name: Name of the new load balancer (required)
+        @type name: C{str}
+
+        @keyword members: C{list} ofL{Member}s to attach to balancer
+        @type: C{list} of L{Member}s
+
+        @keyword protocol: Loadbalancer protocol, defaults to http.
+        @type: C{str}
+
+        @keyword port: Port the load balancer should listen on, defaults to 80
+        @type port: C{str}
+
+        @keyword algorithm: Load balancing algorithm, defaults to
+                            LBAlgorithm.ROUND_ROBIN
+        @type algorithm: C{LBAlgorithm}
+
+        @keyword vip: Virtual ip type of PUBLIC, SERVICENET, or ID of a virtual
+                      ip
+        @type vip: C{str}
+        """
         balancer_attrs = self._kwargs_to_mutable_attrs(
-                                name=name,
-                                protocol=protocol,
-                                port=port,
-                                algorithm=algorithm)
+            name=name,
+            protocol=protocol,
+            port=port,
+            algorithm=algorithm,
+            vip=vip)
 
         balancer_attrs.update({
-            "virtualIps": [{"type": "PUBLIC"}],
-            "nodes": [{"address": member.ip,
-                "port": member.port,
-                "condition": "ENABLED"} for member in members],
+            'nodes': [self._member_attributes(member) for member in members],
             })
         balancer_object = {"loadBalancer": balancer_attrs}
 
         resp = self.connection.request('/loadbalancers',
-                method='POST',
-                data=json.dumps(balancer_object))
-        return self._to_balancer(resp.object["loadBalancer"])
+                                       method='POST',
+                                       data=json.dumps(balancer_object))
+        return self._to_balancer(resp.object['loadBalancer'])
+
+    def _member_attributes(self, member):
+        member_attributes = {'address': member.ip,
+                             'port': member.port}
+
+        member_attributes.update(self._kwargs_to_mutable_member_attrs(
+            **member.extra))
+
+        # If the condition is not specified on the member, then it should be
+        # set to ENABLED by default
+        if 'condition' not in member_attributes:
+            member_attributes['condition'] = \
+            self.CONDITION_LB_MEMBER_MAP[MemberCondition.ENABLED]
+
+        return member_attributes
 
     def destroy_balancer(self, balancer):
         uri = '/loadbalancers/%s' % (balancer.id)
@@ -380,20 +462,31 @@ class RackspaceLBDriver(Driver):
         return self._to_balancer(resp.object["loadBalancer"])
 
     def balancer_attach_member(self, balancer, member):
-        ip = member.ip
-        port = member.port
-
-        member_object = {"nodes":
-                [{"port": port,
-                    "address": ip,
-                    "condition": "ENABLED"}]
-                }
+        member_object = {"nodes": [self._member_attributes(member)]}
 
         uri = '/loadbalancers/%s/nodes' % (balancer.id)
         resp = self.connection.request(uri, method='POST',
                 data=json.dumps(member_object))
         return self._to_members(resp.object)[0]
 
+    def ex_balancer_attach_members(self, balancer, members):
+        """
+        Attaches a list of members to a load balancer.
+
+        @param balancer: The Balancer to which members will be attached.
+        @type balancer: C{Balancer}
+
+        @param members: A list of Members to attach.
+        @type members: C{list}
+        """
+        member_objects = {"nodes": [self._member_attributes(member) for member
+                                    in members]}
+
+        uri = '/loadbalancers/%s/nodes' % (balancer.id)
+        resp = self.connection.request(uri, method='POST',
+                data=json.dumps(member_objects))
+        return self._to_members(resp.object)
+
     def balancer_detach_member(self, balancer, member):
         # Loadbalancer always needs to have at least 1 member.
         # Last member cannot be detached. You can only disable it or destroy
@@ -983,17 +1076,12 @@ class RackspaceLBDriver(Driver):
         balancer = self._get_updated_balancer(balancer)
         access_list = balancer.extra['accessList']
 
-        # LB API does not return the ID for the newly created item, so we have
-        # to fudge it. Rule types and addresses are unique, so this is a safe
-        # way to uniquely identify the new access rule.
-        created_rule = [r for r in access_list \
-                        if rule.rule_type == r.rule_type and \
-                           rule.address == r.address]
+        created_rule = self._find_matching_rule(rule, access_list)
 
         if not created_rule:
             raise LibcloudError('Could not find created rule')
 
-        return created_rule[0]
+        return created_rule
 
     def ex_create_balancer_access_rule_no_poll(self, balancer, rule):
         """
@@ -1017,6 +1105,76 @@ class RackspaceLBDriver(Driver):
 
         return resp.status == httplib.ACCEPTED
 
+    def ex_create_balancer_access_rules(self, balancer, rules):
+        """
+        Adds a list of access rules to a Balancer's access list.  This method
+        blocks until the update request has been processed and the balancer is
+        in a RUNNING state again.
+
+        @param balancer: Balancer to create the access rule for.
+        @type balancer: C{Balancer}
+
+        @param rules: List of C{RackspaceAccessRule} to add to the balancer.
+        @type rules: C{list}
+
+        @rtype: C{RackspaceAccessRule}
+        @return: The created access rules.
+        """
+        accepted = self.ex_create_balancer_access_rules_no_poll(balancer, rules)
+        if not accepted:
+            msg = 'Create access rules not accepted'
+            raise LibcloudError(msg, driver=self)
+
+        balancer = self._get_updated_balancer(balancer)
+        access_list = balancer.extra['accessList']
+
+        created_rules = []
+        for r in rules:
+            matched_rule = self._find_matching_rule(r, access_list)
+            if matched_rule:
+                created_rules.append(matched_rule)
+
+        if len(created_rules) != len(rules):
+            raise LibcloudError('Could not find all created rules')
+
+        return created_rules
+
+    def _find_matching_rule(self, rule_to_find, access_list):
+        """
+        LB API does not return the ID for the newly created rules, so we have
+        to search the list to find the rule with a matching rule type and
+        address to return an object with the right identifier.it.  The API
+        enforces rule type and address uniqueness.
+        """
+        for r in access_list:
+            if rule_to_find.rule_type == r.rule_type and \
+               rule_to_find.address == r.address:
+                return r
+
+        return None
+
+    def ex_create_balancer_access_rules_no_poll(self, balancer, rules):
+        """
+        Adds a list of access rules to a Balancer's access list.  This method
+        returns immediately.
+
+        @param balancer: Balancer to create the access rule for.
+        @type balancer: C{Balancer}
+
+        @param rules: List of C{RackspaceAccessRule} to add to the balancer.
+        @type rules: C{list}
+
+        @rtype: C{bool}
+        @return: Returns whether the create request was accepted.
+        """
+        uri = '/loadbalancers/%s/accesslist' % (balancer.id)
+        resp = self.connection.request(uri, method='POST',
+            data=json.dumps({
+                'accessList' : [rule._to_dict() for rule in rules]
+            }))
+
+        return resp.status == httplib.ACCEPTED
+
     def ex_destroy_balancer_access_rule(self, balancer, rule):
         """
         Removes an access rule from a Balancer's access list.  This method
@@ -1243,6 +1401,12 @@ class RackspaceLBDriver(Driver):
         if "port" in attrs:
             update_attrs['port'] = int(attrs['port'])
 
+        if "vip" in attrs:
+            if attrs['vip'] == 'PUBLIC' or attrs['vip'] == 'SERVICENET':
+                update_attrs['virtualIps'] = [{'type': attrs['vip']}]
+            else:
+                update_attrs['virtualIps'] = [{'id': attrs['vip']}]
+
         return update_attrs
 
     def _kwargs_to_mutable_member_attrs(self, **attrs):
