@@ -40,7 +40,7 @@ from libcloud.storage.types import ObjectDoesNotExistError
 from libcloud.storage.types import ObjectHashMismatchError
 from libcloud.storage.types import InvalidContainerNameError
 from libcloud.common.types import LazyList
-from libcloud.common.openstack import OpenStackBaseConnection
+from libcloud.common.openstack import OpenStackBaseConnection, OpenStackDriverMixin
 
 from libcloud.common.rackspace import (
     AUTH_URL_US, AUTH_URL_UK)
@@ -86,9 +86,11 @@ class CloudFilesResponse(Response):
 
         return data
 
+
 class CloudFilesRawResponse(CloudFilesResponse, RawResponse):
     pass
 
+
 class CloudFilesConnection(OpenStackBaseConnection):
     """
     Base connection class for the Cloudfiles driver.
@@ -97,7 +99,6 @@ class CloudFilesConnection(OpenStackBaseConnection):
     auth_url = AUTH_URL_US
     responseCls = CloudFilesResponse
     rawResponseCls = CloudFilesRawResponse
-    _url_key = "storage_url"
 
     def __init__(self, user_id, key, secure=True, **kwargs):
         super(CloudFilesConnection, self).__init__(user_id, key, secure=secure,
@@ -105,6 +106,31 @@ class CloudFilesConnection(OpenStackBaseConnection):
         self.api_version = API_VERSION
         self.accept_format = 'application/json'
 
+    def get_endpoint(self, cdn_request=False):
+
+        # First, we parse out both files and cdn endpoints for each auth version
+        if '2.0' in self._auth_version:
+            ep = self.service_catalog.get_endpoint(service_type='object-store',
+                                                   name='cloudFiles',
+                                                   region='ORD')
+            cdn_ep = self.service_catalog.get_endpoint(service_type='object-store',
+                                                       name='cloudFilesCDN',
+                                                       region='ORD')
+        elif ('1.1' in self._auth_version) or ('1.0' in self._auth_version):
+            ep = self.service_catalog.get_endpoint(name='cloudFiles',
+                                                   region='ORD')
+            cdn_ep = self.service_catalog.get_endpoint(name='cloudFilesCDN',
+                                                       region='ORD')
+
+        # if this is a CDN request, return the cdn url instead
+        if cdn_request:
+            ep = cdn_ep
+
+        if 'publicURL' in ep:
+            return ep['publicURL']
+        else:
+            raise LibcloudError('Could not find specified endpoint')
+
     def request(self, action, params=None, data='', headers=None, method='GET',
                 raw=False, cdn_request=False):
         if not headers:
@@ -112,14 +138,19 @@ class CloudFilesConnection(OpenStackBaseConnection):
         if not params:
             params = {}
 
-        if cdn_request:
-            host = self._get_host(url_key='cdn_management_url')
+        # FIXME: Massive hack.
+        # This driver dynamically changes the url in its connection, based on arguments
+        # passed to request(). As such, we have to manually check and reset connection
+        # params each request
+        self._populate_hosts_and_request_paths()
+        if not self._ex_force_base_url:
+            self._reset_connection_params(self.get_endpoint(cdn_request))
         else:
-            host = None
+            self._reset_connection_params(self._ex_force_base_url)
 
         params['format'] = 'json'
 
-        if method in [ 'POST', 'PUT' ]:
+        if method in ['POST', 'PUT']:
             headers.update({'Content-Type': 'application/json; charset=UTF-8'})
 
         return super(CloudFilesConnection, self).request(
@@ -128,6 +159,9 @@ class CloudFilesConnection(OpenStackBaseConnection):
             method=method, headers=headers,
             raw=raw)
 
+    def _reset_connection_params(self, endpoint_url):
+        (self.host, self.port, self.secure, self.request_path) = self._tuple_from_url(endpoint_url)
+
 
 class CloudFilesUSConnection(CloudFilesConnection):
     """
@@ -145,7 +179,7 @@ class CloudFilesUKConnection(CloudFilesConnection):
     auth_url = AUTH_URL_UK
 
 
-class CloudFilesStorageDriver(StorageDriver):
+class CloudFilesStorageDriver(StorageDriver, OpenStackDriverMixin):
     """
     Base CloudFiles driver.
 
@@ -158,6 +192,10 @@ class CloudFilesStorageDriver(StorageDriver):
     hash_type = 'md5'
     supports_chunked_encoding = True
 
+    def __init__(self, *args, **kwargs):
+        OpenStackDriverMixin.__init__(self, *args, **kwargs)
+        super(CloudFilesStorageDriver, self).__init__(*args, **kwargs)
+
     def list_containers(self):
         response = self.connection.request('')
 
@@ -510,6 +548,9 @@ class CloudFilesStorageDriver(StorageDriver):
                      meta_data=meta_data, container=container, driver=self)
         return obj
 
+    def _ex_connection_class_kwargs(self):
+        return self.openstack_connection_kwargs()
+
 class CloudFilesUSStorageDriver(CloudFilesStorageDriver):
     """
     Cloudfiles storage driver for the US endpoint.
