@@ -73,6 +73,9 @@ class ObjectError(LibcloudError):
         super(ObjectError, self).__init__(value=value, driver=driver)
 
     def __str__(self):
+        return self.__repr__()
+
+    def __repr__(self):
         return '<%s in %s, value=%s, object = %s>' % (self.error_type, repr(self.driver),
                                                       self.value, self.object_name)
 
