@@ -13,14 +13,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import httplib
 import random
 import unittest
 
-from cStringIO import StringIO
-from urllib2 import urlparse
 from cgi import parse_qs
 
+from libcloud.utils.py3 import httplib
+from libcloud.utils.py3 import StringIO
+from libcloud.utils.py3 import urlparse
+from libcloud.utils.py3 import u
+
+
 XML_HEADERS = {'content-type': 'application/xml'}
 
 
@@ -75,7 +78,7 @@ class MockResponse(object):
 
     def __init__(self, status, body, headers=None, reason=None):
         self.status = status
-        self.body = StringIO(body)
+        self.body = StringIO(u(body))
         self.headers = headers or self.headers
         self.reason = reason or self.reason
 
@@ -86,7 +89,7 @@ class MockResponse(object):
         return self.headers.get(name, *args, **kwargs)
 
     def getheaders(self):
-        return self.headers.items()
+        return list(self.headers.items())
 
     def msg(self):
         raise NotImplemented
@@ -240,6 +243,9 @@ class MockRawResponse(BaseMockHttpObject):
         self._current_item += 1
         return value
 
+    def __next__(self):
+        return self.next()
+
     def _generate_random_data(self, size):
         data = []
         current_size = 0
