@@ -23,7 +23,6 @@ except ImportError:
     import json
 
 from libcloud.utils.py3 import httplib
-from libcloud.utils.py3 import urllib
 from libcloud.utils.py3 import urlencode
 
 from libcloud.loadbalancer.base import LoadBalancer, Member, Algorithm
@@ -45,6 +44,53 @@ class RackspaceLBTests(unittest.TestCase):
         RackspaceLBMockHttp.type = None
         self.driver = RackspaceLBDriver('user', 'key')
         self.driver.connection.poll_interval = 0.0
+        # normally authentication happens lazily, but we force it here
+        self.driver.connection._populate_hosts_and_request_paths()
+
+    def test_force_auth_token_kwargs(self):
+        base_url = 'https://ord.loadbalancer.api.rackspacecloud.com/v1.0/slug'
+        kwargs = {
+            'ex_force_auth_token': 'some-auth-token',
+            'ex_force_base_url': base_url
+        }
+        driver = RackspaceLBDriver('user', 'key', **kwargs)
+        driver.list_balancers()
+
+        self.assertEquals(kwargs['ex_force_auth_token'],
+            driver.connection.auth_token)
+        self.assertEquals('/v1.0/slug',
+            driver.connection.request_path)
+
+    def test_force_auth_url_kwargs(self):
+        kwargs = {
+            'ex_force_auth_version': '2.0',
+            'ex_force_auth_url': 'https://identity.api.rackspace.com'
+        }
+        driver = RackspaceLBDriver('user', 'key', **kwargs)
+
+        self.assertEquals(kwargs['ex_force_auth_url'],
+            driver.connection._ex_force_auth_url)
+        self.assertEquals(kwargs['ex_force_auth_version'],
+            driver.connection._auth_version)
+
+    def test_gets_auth_2_0_endpoint_defaults_to_ord_region(self):
+        driver = RackspaceLBDriver('user', 'key',
+            ex_force_auth_version='2.0_password'
+        )
+        driver.connection._populate_hosts_and_request_paths()
+
+        self.assertEquals('https://ord.loadbalancers.api.rackspacecloud.com/v1.0/11111',
+            driver.connection.get_endpoint())
+
+    def test_gets_auth_2_0_endpoint_for_dfw(self):
+        driver = RackspaceLBDriver('user', 'key',
+            ex_force_auth_version='2.0_password',
+            ex_force_region='dfw'
+        )
+        driver.connection._populate_hosts_and_request_paths()
+
+        self.assertEquals('https://dfw.loadbalancers.api.rackspacecloud.com/v1.0/11111',
+            driver.connection.get_endpoint())
 
     def test_list_protocols(self):
         protocols = self.driver.list_protocols()
@@ -103,13 +149,29 @@ class RackspaceLBTests(unittest.TestCase):
         balancer = self.driver.create_balancer(name='test2',
                 port=80,
                 algorithm=Algorithm.ROUND_ROBIN,
-                members=(Member(None, '10.1.0.10', 80),
-                    Member(None, '10.1.0.11', 80))
+                members=(Member(None, '10.1.0.10', 80, extra={'condition': MemberCondition.DISABLED,
+                                                              'weight': 10}),
+                         Member(None, '10.1.0.11', 80))
                 )
 
         self.assertEquals(balancer.name, 'test2')
         self.assertEquals(balancer.id, '8290')
 
+    def test_ex_create_balancer(self):
+        RackspaceLBDriver.connectionCls.conn_classes = (None,
+                                                        RackspaceLBWithVIPMockHttp)
+        RackspaceLBMockHttp.type = None
+        driver = RackspaceLBDriver('user', 'key')
+        balancer = driver.ex_create_balancer(name='test2',
+                port=80,
+                algorithm=Algorithm.ROUND_ROBIN,
+                members=(Member(None, '10.1.0.11', 80),),
+                vip='12af'
+        )
+
+        self.assertEquals(balancer.name, 'test2')
+        self.assertEquals(balancer.id, '8290')
+
     def test_destroy_balancer(self):
         balancer = self.driver.list_balancers()[0]
 
@@ -314,6 +376,33 @@ class RackspaceLBTests(unittest.TestCase):
 
         self.assertTrue(resp)
 
+    def test_ex_create_balancer_access_rules(self):
+        balancer = self.driver.get_balancer(balancer_id='94699')
+
+        rules = [RackspaceAccessRule(rule_type=RackspaceAccessRuleType.ALLOW,
+                                     address='2001:4801:7901::6/64'),
+                 RackspaceAccessRule(rule_type=RackspaceAccessRuleType.DENY,
+                                     address='8.8.8.8/0')]
+
+        rules = self.driver.ex_create_balancer_access_rules(balancer, rules)
+
+        self.assertEquals(2, len(rules))
+        self.assertEquals(2884, rules[0].id)
+        self.assertEquals(3006, rules[1].id)
+
+    def test_ex_create_balancer_access_rules_no_poll(self):
+        balancer = self.driver.get_balancer(balancer_id='94699')
+
+        rules = [RackspaceAccessRule(rule_type=RackspaceAccessRuleType.ALLOW,
+                                     address='2001:4801:7901::6/64'),
+                 RackspaceAccessRule(rule_type=RackspaceAccessRuleType.DENY,
+                                     address='8.8.8.8/0')]
+
+        resp = self.driver.ex_create_balancer_access_rules_no_poll(balancer,
+            rules)
+
+        self.assertTrue(resp)
+
     def test_ex_destroy_balancer_access_rule(self):
         balancer = self.driver.get_balancer(balancer_id='94698')
 
@@ -322,7 +411,7 @@ class RackspaceLBTests(unittest.TestCase):
             address="10.45.13.5/12"
         )
 
-        balancer= self.driver.ex_destroy_balancer_access_rule(balancer, rule)
+        balancer = self.driver.ex_destroy_balancer_access_rule(balancer, rule)
 
         rule_ids = [r.id for r in balancer.extra['accessList']]
 
@@ -596,12 +685,37 @@ class RackspaceLBTests(unittest.TestCase):
 
     def test_balancer_attach_member(self):
         balancer = self.driver.get_balancer(balancer_id='8290')
+        extra = {'condition': MemberCondition.DISABLED,
+                 'weight': 10}
+        member = balancer.attach_member(Member(None, ip='10.1.0.12',
+                                               port='80', extra=extra))
+
+        self.assertEquals(member.ip, '10.1.0.12')
+        self.assertEquals(member.port, 80)
+
+    def test_balancer_attach_member_with_no_condition_specified(self):
+        balancer = self.driver.get_balancer(balancer_id='8291')
         member = balancer.attach_member(Member(None, ip='10.1.0.12',
                                                port='80'))
 
         self.assertEquals(member.ip, '10.1.0.12')
         self.assertEquals(member.port, 80)
 
+    def test_balancer_attach_members(self):
+        balancer = self.driver.get_balancer(balancer_id='8292')
+        members = [Member(None, ip='10.1.0.12', port='80'),
+                   Member(None, ip='10.1.0.13', port='80')]
+
+        attached_members = self.driver.ex_balancer_attach_members(balancer,
+                                                                  members)
+
+        first_member = attached_members[0]
+        second_member = attached_members[1]
+        self.assertEquals(first_member.ip, '10.1.0.12')
+        self.assertEquals(first_member.port, 80)
+        self.assertEquals(second_member.ip, '10.1.0.13')
+        self.assertEquals(second_member.port, 80)
+
     def test_balancer_detach_member(self):
         balancer = self.driver.get_balancer(balancer_id='8290')
         member = balancer.list_members()[0]
@@ -764,6 +878,9 @@ class RackspaceUKLBTests(RackspaceLBTests):
                 RackspaceLBMockHttp)
         RackspaceLBMockHttp.type = None
         self.driver = RackspaceUKLBDriver('user', 'key')
+        # normally authentication happens lazily, but we force it here
+        self.driver.connection._populate_hosts_and_request_paths()
+
 
 class RackspaceLBMockHttp(MockHttpTestCase):
     fixtures = LoadBalancerFileFixtures('rackspace')
@@ -796,8 +913,16 @@ class RackspaceLBMockHttp(MockHttpTestCase):
             return (httplib.OK, body, {}, httplib.responses[httplib.OK])
         elif method == "POST":
             body_json = json.loads(body)
-            self.assertEqual(body_json['loadBalancer']['protocol'], 'HTTP')
-            self.assertEqual(body_json['loadBalancer']['algorithm'], 'ROUND_ROBIN')
+            loadbalancer_json = body_json['loadBalancer']
+            member_1_json, member_2_json = loadbalancer_json['nodes']
+
+            self.assertEqual(loadbalancer_json['protocol'], 'HTTP')
+            self.assertEqual(loadbalancer_json['algorithm'], 'ROUND_ROBIN')
+            self.assertEqual(loadbalancer_json['virtualIps'][0]['type'],
+                             'PUBLIC')
+            self.assertEqual(member_1_json['condition'], 'DISABLED')
+            self.assertEqual(member_1_json['weight'], 10)
+            self.assertEqual(member_2_json['condition'], 'ENABLED')
 
             body = self.fixtures.load('v1_slug_loadbalancers_post.json')
             return (httplib.ACCEPTED, body, {},
@@ -837,8 +962,12 @@ class RackspaceLBMockHttp(MockHttpTestCase):
             body = self.fixtures.load('v1_slug_loadbalancers_8290_nodes.json')
             return (httplib.OK, body, {}, httplib.responses[httplib.OK])
         elif method == "POST":
-            body = self.fixtures.load('v1_slug_loadbalancers_8290_nodes_post.json')
-            return (httplib.ACCEPTED, body, {},
+            json_body = json.loads(body)
+            json_node = json_body['nodes'][0]
+            self.assertEqual('DISABLED', json_node['condition'])
+            self.assertEqual(10, json_node['weight'])
+            response_body = self.fixtures.load('v1_slug_loadbalancers_8290_nodes_post.json')
+            return (httplib.ACCEPTED, response_body, {},
                     httplib.responses[httplib.ACCEPTED])
         elif method == "DELETE":
             nodes = self.fixtures.load('v1_slug_loadbalancers_8290_nodes.json')
@@ -853,6 +982,44 @@ class RackspaceLBMockHttp(MockHttpTestCase):
 
         raise NotImplementedError
 
+    def _v1_0_slug_loadbalancers_8291(self, method, url, body, headers):
+        if method == "GET":
+            body = self.fixtures.load('v1_slug_loadbalancers_8291.json')
+            return (httplib.OK, body, {}, httplib.responses[httplib.OK])
+
+        raise NotImplementedError
+
+    def _v1_0_slug_loadbalancers_8291_nodes(self, method, url, body, headers):
+        if method == "POST":
+            json_body = json.loads(body)
+            json_node = json_body['nodes'][0]
+            self.assertEqual('ENABLED', json_node['condition'])
+            response_body = self.fixtures.load('v1_slug_loadbalancers_8290_nodes_post.json')
+            return (httplib.ACCEPTED, response_body, {},
+                    httplib.responses[httplib.ACCEPTED])
+
+        raise NotImplementedError
+
+    def _v1_0_slug_loadbalancers_8292(self, method, url, body, headers):
+        if method == "GET":
+            body = self.fixtures.load('v1_slug_loadbalancers_8292.json')
+            return (httplib.OK, body, {}, httplib.responses[httplib.OK])
+
+        raise NotImplementedError
+
+    def _v1_0_slug_loadbalancers_8292_nodes(self, method, url, body, headers):
+        if method == "POST":
+            json_body = json.loads(body)
+            json_node_1 = json_body['nodes'][0]
+            json_node_2 = json_body['nodes'][1]
+            self.assertEqual('10.1.0.12', json_node_1['address'])
+            self.assertEqual('10.1.0.13', json_node_2['address'])
+            response_body = self.fixtures.load('v1_slug_loadbalancers_8292_nodes_post.json')
+            return (httplib.ACCEPTED, response_body, {},
+                    httplib.responses[httplib.ACCEPTED])
+
+        raise NotImplementedError
+
     def _v1_0_slug_loadbalancers_8290_nodes_30944(self, method, url, body, headers):
         if method == "PUT":
             json_body = json.loads(body)
@@ -1100,6 +1267,16 @@ class RackspaceLBMockHttp(MockHttpTestCase):
                     msg='Did not delete access rule with id %d' % id)
 
             return (httplib.ACCEPTED, '', {}, httplib.responses[httplib.ACCEPTED])
+        elif method == 'POST':
+
+            json_body = json.loads(body)
+            access_list = json_body['accessList']
+            self.assertEquals('ALLOW', access_list[0]['type'])
+            self.assertEquals('2001:4801:7901::6/64', access_list[0]['address'])
+            self.assertEquals('DENY', access_list[1]['type'])
+            self.assertEquals('8.8.8.8/0', access_list[1]['address'])
+
+            return (httplib.ACCEPTED, '', {}, httplib.responses[httplib.ACCEPTED])
 
         raise NotImplementedError
 
@@ -1204,5 +1381,55 @@ class RackspaceLBMockHttp(MockHttpTestCase):
         body = self.auth_fixtures.load('_v1_1__auth.json')
         return (httplib.OK, body, headers, httplib.responses[httplib.OK])
 
+    def _v2_0_tokens(self, method, url, body, headers):
+        body = self.fixtures.load('auth_2_0.json')
+        headers = {
+            'content-type': 'application/json'
+        }
+        return (httplib.OK, body, headers,
+                httplib.responses[httplib.OK])
+
+
+class RackspaceLBWithVIPMockHttp(MockHttpTestCase):
+    fixtures = LoadBalancerFileFixtures('rackspace')
+    auth_fixtures = OpenStackFixtures()
+
+    def _v1_0(self, method, url, body, headers):
+        headers = {'x-server-management-url': 'https://servers.api.rackspacecloud.com/v1.0/slug',
+                   'x-auth-token': 'FE011C19-CF86-4F87-BE5D-9229145D7A06',
+                   'x-cdn-management-url': 'https://cdn.clouddrive.com/v1/MossoCloudFS_FE011C19-CF86-4F87-BE5D-9229145D7A06',
+                   'x-storage-token': 'FE011C19-CF86-4F87-BE5D-9229145D7A06',
+                   'x-storage-url': 'https://storage4.clouddrive.com/v1/MossoCloudFS_FE011C19-CF86-4F87-BE5D-9229145D7A06'}
+        return (httplib.NO_CONTENT, "", headers, httplib.responses[httplib.NO_CONTENT])
+
+    def _v1_0_slug_loadbalancers(self, method, url, body, headers):
+        if method == "GET":
+            body = self.fixtures.load('v1_slug_loadbalancers.json')
+            return (httplib.OK, body, {}, httplib.responses[httplib.OK])
+        elif method == "POST":
+            body_json = json.loads(body)
+            loadbalancer_json = body_json['loadBalancer']
+
+            self.assertEqual(loadbalancer_json['virtualIps'][0]['id'], '12af')
+
+            body = self.fixtures.load('v1_slug_loadbalancers_post.json')
+            return (httplib.ACCEPTED, body, {},
+                    httplib.responses[httplib.ACCEPTED])
+
+        raise NotImplementedError
+
+    def _v1_1_auth(self, method, url, body, headers):
+        headers = {'content-type': 'application/json; charset=UTF-8'}
+        body = self.auth_fixtures.load('_v1_1__auth.json')
+        return (httplib.OK, body, headers, httplib.responses[httplib.OK])
+
+    def _v2_0_tokens(self, method, url, body, headers):
+        body = self.fixtures.load('auth_2_0.json')
+        headers = {
+            'content-type': 'application/json'
+        }
+        return (httplib.OK, body, headers,
+                httplib.responses[httplib.OK])
+
 if __name__ == "__main__":
     sys.exit(unittest.main())
