@@ -42,6 +42,7 @@ from test.file_fixtures import StorageFileFixtures, OpenStackFixtures # pylint:
 
 current_hash = None
 
+
 class CloudFilesTests(unittest.TestCase):
 
     def setUp(self):
@@ -52,11 +53,41 @@ class CloudFilesTests(unittest.TestCase):
         CloudFilesMockHttp.type = None
         CloudFilesMockRawResponse.type = None
         self.driver = CloudFilesStorageDriver('dummy', 'dummy')
+        # normally authentication happens lazily, but we force it here
+        self.driver.connection._populate_hosts_and_request_paths()
         self._remove_test_file()
 
     def tearDown(self):
         self._remove_test_file()
 
+    def test_force_auth_token_kwargs(self):
+        base_url = 'https://cdn2.clouddrive.com/v1/MossoCloudFS'
+        kwargs = {
+            'ex_force_auth_token': 'some-auth-token',
+            'ex_force_base_url': base_url
+        }
+        driver = CloudFilesStorageDriver('driver', 'dummy', **kwargs)
+        driver.list_containers()
+
+        self.assertEquals(kwargs['ex_force_auth_token'],
+            driver.connection.auth_token)
+        self.assertEquals('cdn2.clouddrive.com',
+            driver.connection.host)
+        self.assertEquals('/v1/MossoCloudFS',
+            driver.connection.request_path)
+
+    def test_force_auth_url_kwargs(self):
+        kwargs = {
+            'ex_force_auth_version': '2.0',
+            'ex_force_auth_url': 'https://identity.api.rackspace.com'
+        }
+        driver = CloudFilesStorageDriver('driver', 'dummy', **kwargs)
+
+        self.assertEquals(kwargs['ex_force_auth_url'],
+            driver.connection._ex_force_auth_url)
+        self.assertEquals(kwargs['ex_force_auth_version'],
+            driver.connection._auth_version)
+
     def test_invalid_json_throws_exception(self):
         CloudFilesMockHttp.type = 'MALFORMED_JSON'
         try:
@@ -162,7 +193,7 @@ class CloudFilesTests(unittest.TestCase):
                 'Container already exists but an exception was not thrown')
 
     def test_create_container_invalid_name_too_long(self):
-        name = ''.join([ 'x' for x in range(0, 257)])
+        name = ''.join(['x' for x in range(0, 257)])
         try:
             self.driver.create_container(container_name=name)
         except InvalidContainerNameError:
@@ -272,7 +303,7 @@ class CloudFilesTests(unittest.TestCase):
         file_path = os.path.abspath(__file__)
         container = Container(name='foo_bar_container', extra={}, driver=self)
         object_name = 'foo_test_upload'
-        extra = {'meta_data': { 'some-value': 'foobar'}}
+        extra = {'meta_data': {'some-value': 'foobar'}}
         obj = self.driver.upload_object(file_path=file_path, container=container,
                                         extra=extra, object_name=object_name)
         self.assertEqual(obj.name, 'foo_test_upload')
